from .icon_checkbox import IconCheckBox
from .misc import (
    ValidateFloatVar,
    error_bar_visibility,
    flatten_dict,
    fpe,
    update_errorbar,
)
from .toolbar import VerticalNavigationToolbar2Tk



import tkinter as tk

import customtkinter as ctk
import numpy as np
from matplotlib import cbook
from matplotlib.backend_bases import NavigationToolbar2
from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk
from PIL import Image

from .icon_checkbox import IconCheckBox


class VerticalNavigationToolbar2Tk(NavigationToolbar2Tk):
    def __init__(self, canvas, window, pack_toolbar=False, **kwargs):
        NavigationToolbar2Tk._set_image_for_button = self._set_image_for_button

        super().__init__(canvas, window, pack_toolbar=False, **kwargs)

    # # Override the damn image selection
    @staticmethod
    def _set_image_for_button(self, button):
        """
        Set the image for a button based on its pixel size.

        The pixel size is determined by the DPI scaling of the window.
        """

        if button._image_file is None:
            return

        # Allow _image_file to be relative to Matplotlib's "images" data
        # directory.
        path_regular = cbook._get_data_path("images", button._image_file)
        path_large = path_regular.with_name(
            path_regular.name.replace(".png", "_large.png")
        )
        size = button.winfo_pixels("18p")

        # Nested functions because ToolbarTk calls  _Button.
        def _get_color(color_name):
            # `winfo_rgb` returns an (r, g, b) tuple in the range 0-65535
            return button.winfo_rgb(button.cget(color_name))

        def _is_dark(color):
            if isinstance(color, str):
                color = _get_color(color)
            return max(color) < 65535 / 2

        def _recolor_icon(image, color):
            image_data = np.asarray(image).copy()
            black_mask = (image_data[..., :3] == 0).all(axis=-1)
            image_data[black_mask, :3] = color
            return Image.fromarray(image_data, mode="RGBA")

        # Use the high-resolution (48x48 px) icon if it exists and is needed
        with Image.open(
            path_large if (size > 24 and path_large.exists()) else path_regular
        ) as im:
            # assure a RGBA image as foreground color is RGB
            im = im.convert("RGBA")
            image = ctk.CTkImage(im.resize((size, size)))
            button._ntimage = image

            # create a version of the icon with the button's text color
            foreground = 255 * np.array(self._root().text_colour)
            im_alt = _recolor_icon(im, foreground)
            image_alt = ctk.CTkImage(im_alt.resize((size, size)))
            button._ntimage_alt = image_alt

        if ctk.get_appearance_mode() == "Dark":
            image_kwargs = {"image": image_alt}
        else:
            image_kwargs = {"image": image}

        # Checkbuttons may switch the background to `selectcolor` in the
        # checked state, so check separately which image it needs to use in
        # that state to still ensure enough contrast with the background.
        if isinstance(button, tk.Checkbutton) and button.cget("selectcolor") != "":
            if self._windowingsystem != "x11":
                selectcolor = "selectcolor"
            else:
                # On X11, selectcolor isn't used directly for indicator-less
                # buttons. See `::tk::CheckEnter` in the Tk button.tcl source
                # code for details.
                r1, g1, b1 = _get_color("selectcolor")
                r2, g2, b2 = _get_color("activebackground")
                selectcolor = ((r1 + r2) / 2, (g1 + g2) / 2, (b1 + b2) / 2)
            if _is_dark(selectcolor):
                image_kwargs["selectimage"] = image_alt
            else:
                image_kwargs["selectimage"] = image

        try:
            button.configure(**image_kwargs, height=18, width=18)
        except:
            button.configure(
                unchecked_icon=image_kwargs["image"],
                checked_icon=image_kwargs["image"],
                height=18,
                width=18,
            )
            button._update_image()
            button._draw()
            pass

    # override _Button() to re-pack the toolbar button in vertical direction
    def _Button(self, text, image_file, toggle, command):
        if not toggle:
            b = ctk.CTkButton(
                master=self,
                text=None,
                command=command,
                bg_color=self._root().bg_colour_name,
                fg_color=self._root().bg_colour_name,
            )
        else:
            # There is a bug in tkinter included in some python 3.6 versions
            # that without this variable, produces a "visual" toggling of
            # other near checkbuttons
            # https://bugs.python.org/issue29402
            # https://bugs.python.org/issue25684
            var = ctk.IntVar(master=self)
            b = IconCheckBox(
                master=self,
                text="",
                command=command,
                unchecked_icon=image_file,
                checked_icon=image_file,
                fg_color=ctk.ThemeManager.theme["CTkButton"]["fg_color"],
                variable=var,
                bg_color=self._root().bg_colour_name,
                auto_dark_icon=True,
            )
            b.var = var
        b._image_file = image_file
        if image_file is not None:
            # Explicit class because ToolbarTk calls _Button.
            NavigationToolbar2Tk._set_image_for_button(self, b)
        else:
            b.configure(font=self._label_font)
        b.pack(side="top", pady=5)  # re-pack button in vertical direction
        return b

    # override _Spacer() to create vertical separator
    def _Spacer(self):
        s = tk.Frame(self, width=26, relief="ridge", bg="DarkGray", padx=2)
        s.pack(side="top", pady=5)  # pack in vertical direction
        return s

    # disable showing mouse position in toolbar
    def set_message(self, s):
        pass



import re

import astropy.units as u
import customtkinter as ctk
import numpy as np
from astropy.coordinates import SkyCoord
from CTkMessagebox import CTkMessagebox

from pygcg.utils import ValidateFloatVar

from .base_window import BaseWindow


class SearchWindow(BaseWindow):
    def __init__(self, *args, **kwargs):
        super().__init__(
            *args,
            title="Find Multiple Objects",
            top_text="Insert each object on a new line.",
            **kwargs,
        )
        self.action_button.configure(text="Search")
        self.top_label.grid_configure(columnspan=1)

        radius_frame = ctk.CTkFrame(
            self.main_frame, fg_color=self._root().bg_colour_name
        )
        radius_frame.grid(row=0, column=1, sticky="ew")
        radius_label = ctk.CTkLabel(radius_frame, text="Search radius (arcsec):")
        radius_label.grid(row=0, column=0, padx=(10, 5), pady=(10, 0), sticky="e")

        self.search_radius = ValidateFloatVar(self)
        self.search_radius.set(1.0)
        self.radius_entry = ctk.CTkEntry(radius_frame, textvariable=self.search_radius)
        self.radius_entry.grid(row=0, column=1, padx=(5, 10), pady=(10, 0), sticky="w")

    def warn_input(self, exception=None):
        error = CTkMessagebox(
            title="Error",
            message=exception,
            icon="cancel",
            option_focus=1,
        )
        if error.get() == "OK":
            self.focus_force()
            return

    def action_button_callback(self, event=None):
        if hasattr(self, "ids_arr"):
            del self.ids_arr

        text_input = self.text_box.get("1.0", "end")

        lines = re.split("\n", text_input.strip())

        try:
            parts = re.split(r"\s*[,|;|\s]\s*", lines[0].strip())
        except Exception as e:
            self.warn_input(f"Could not parse input: {e}")
            return

        ras = np.empty(len(lines), dtype=object)
        decs = np.empty_like(ras, dtype=object)
        self.ids_arr = np.empty_like(ras, dtype=object)

        if len(parts) == 1:
            for i, l in enumerate(lines):
                self.ids_arr[i] = re.split(r"\s*[,|;|\s]\s*", l.strip())[0]
            self.ids_arr = np.asarray(self.ids_arr)
            _, _, match_idx = np.intersect1d(
                np.asarray(self.ids_arr), self._root().id_col, return_indices=True
            )

        else:
            try:

                if len(parts) == 3:
                    # self.ids_arr = np.empty_like(ras, dtype=object)
                    for i, l in enumerate(lines):
                        # print(re.split("\s*[,|;|\s]\s*", l.strip()))
                        self.ids_arr[i], ras[i], decs[i] = re.split(
                            r"\s*[,|;|\s]\s*", l.strip()
                        )

                elif len(parts) == 2:
                    for i, l in enumerate(lines):
                        # print(re.split("\s*[,|;|\s]\s*", l.strip()))
                        ras[i], decs[i] = re.split(r"\s*[,|;|\s]\s*", l.strip())
                else:
                    raise ValueError()
            except Exception as e:
                self.warn_input(
                    "Could not parse input: input must be either "
                    "two or three components per line."
                )
                return

            try:
                new_coords = SkyCoord(
                    ras.astype(float) * u.deg, decs.astype(float) * u.deg
                )
            except Exception as e:
                try:
                    new_coords = SkyCoord(ras, decs)
                except Exception as e:
                    try:
                        new_coords = SkyCoord(ras, decs, unit=(u.hourangle, u.deg))
                    except Exception as e:
                        self.warn_input(f"Could not parse input coordinates: {e}")
                        return

            sky_match_idx, dist, _ = new_coords.match_to_catalog_sky(
                self._root().sky_coords
            )

            sky_match_idx = sky_match_idx[
                dist <= float(self.search_radius.get()) * u.arcsec
            ]
            if hasattr(self, "ids_arr"):
                self.ids_arr = self.ids_arr[
                    dist <= float(self.search_radius.get()) * u.arcsec
                ]

            _, unique_idx = np.unique(sky_match_idx, return_index=True)
            match_idx = sky_match_idx[np.sort(unique_idx)]
            if hasattr(self, "ids_arr"):
                self.ids_arr = self.ids_arr[np.sort(unique_idx)]

        if len(match_idx) == 0:
            self.warn_input("No matches found!")
            return

        check_confirm = CTkMessagebox(
            title="Confirm Selection",
            message=(
                f"{len(match_idx)}/{len(ras)} entries have a unique match in the "
                "current catalogue. View only these objects?"
            ),
            icon="question",
            option_1="Cancel",
            option_2="Confirm",
            option_focus=2,
        )
        if check_confirm.get() == "Cancel":
            self.focus_force()
            return

        self._root().id_col = self.ids_arr
        self._root().seg_id_col = self._root().seg_id_col[match_idx]
        self._root().cat = self._root().cat[match_idx]
        self._root().sky_coords = SkyCoord(
            self._root().cat[
                self._root().config.get("catalogue", {}).get("ra", "X_WORLD")
            ],
            self._root().cat[
                self._root().config.get("catalogue", {}).get("dec", "Y_WORLD")
            ],
        )

        self._root().current_gal_id.set(self._root().id_col[0])
        self._root().tab_row = self._root().cat[0]
        self._root().seg_id = self._root().seg_id_col[0]
        if hasattr(self._root(), "current_seg_id"):
            self._root().current_seg_id.set(self._root().seg_id)

        self._root().set_current_data()

        self._root().generate_tabs()
